==> smile_flutter-0.0.1.tar.gz: android/src/main/kotlin/com/smileidentity/smile_flutter/SmileFlutterPlugin.kt <==
package com.smileidentity.smile_flutter

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.ConnectivityManager
import android.os.Build
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import androidx.annotation.Nullable
import androidx.core.app.ActivityCompat
import com.rnsmileid.SIDUtil
import com.smileid.smileidui.CaptureType
import com.smileid.smileidui.IntentHelper.SELFIE_REQUEST_RESULT
import com.smileid.smileidui.IntentHelper.SMILE_REQUEST_RESULT_TAG
import com.smileid.smileidui.SIDCaptureManager
import com.smileidentity.libsmileid.core.SIDConfig
import com.smileidentity.libsmileid.core.SIDNetworkRequest
import com.smileidentity.libsmileid.model.GeoInfos
import com.smileidentity.libsmileid.model.SIDMetadata
import com.smileidentity.libsmileid.model.SIDNetData

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.RequestPermissionsResultListener
import org.json.JSONObject

/** SmileFlutterPlugin */
class SmileFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    PluginRegistry.ActivityResultListener, RequestPermissionsResultListener {


    private val uploadListenerEventChannel = "smile_id/upload_listener"
    private val completeListenerEventChannel = "smile_id/complete_listener"

    private val PERMISSION_ALL = 1
    private val SMILE_ID_CARD_REQUEST_CODE = 777
    val SID_RESULT_CODE = "SID_RESULT_CODE"
    val SID_RESULT_MESSAGE = "SID_RESULT_MESSAGE"
    val SID_RESULT_TAG = "SID_RESULT_TAG"

    val mainIdInfo =
        listOf("first_name", "last_name", "middle_name", "country", "id_type", "id_number", "email")
    val mainPartnerParams = listOf("user_id", "job_id")
    val permissionError = "permissionError"
    val submitError = "submitError"

    val permissionErrorMessage = "Does not have required permissions to run this method"

    private var PERMISSIONS = arrayOf(
        Manifest.permission.WRITE_EXTERNAL_STORAGE,
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.CAMERA,
        Manifest.permission.ACCESS_FINE_LOCATION,
        Manifest.permission.READ_PHONE_STATE
    )

    private lateinit var activity: Activity
    private lateinit var activityBinding: ActivityPluginBinding


    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var currentResult: Result

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "smile_flutter")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        currentResult = result
        when (call.method) {
            "getPlatformVersion" -> {
                currentResult.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "captureSelfie" -> {
                if (!checkPermissions()) {
                    currentResult.error(permissionError, permissionErrorMessage, null)
                    return
                }

                smileCapture(call.argument<String>("tag"), CaptureType.SELFIE)
            }
            "captureIDCard" -> {
                if (!checkPermissions()) {
                    currentResult.error(permissionError, permissionErrorMessage, null)
                    return
                }
                smileCapture(call.argument<String>("tag"), CaptureType.ID_CAPTURE)
            }
            "captureSelfieAndIDCard" -> {
                if (!checkPermissions()) {
                    currentResult.error(permissionError, permissionErrorMessage, null)
                    return
                }
                smileCapture(call.argument<String>("tag"), CaptureType.SELFIE_AND_ID_CAPTURE)
            }
            "submitJob" -> {
                submitJob(
                    call.argument<String>("tag"),
                    call.argument<Int>("jobType"),
                    call.argument<Boolean>("isProduction"),
                    call.argument<HashMap<String, String>>("partnerParams"),
                    call.argument<HashMap<String, String>>("idInfo"),
                    call.argument<HashMap<String, String>>("geoInfo")
                )
            }
            else -> {
                currentResult.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }


    //smile id

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityBinding = binding
        activity = binding.activity
        binding.addRequestPermissionsResultListener(this);
        binding.addActivityResultListener(this);
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activityBinding.removeActivityResultListener(this)
        activityBinding.removeRequestPermissionsResultListener(this)
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
        activityBinding.removeRequestPermissionsResultListener(this);
        activityBinding.removeActivityResultListener(this);
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        val map: HashMap<String, Any> = HashMap()
        data?.extras?.getInt(SELFIE_REQUEST_RESULT)?.let { map.put(SID_RESULT_CODE, it) }
        data?.extras?.getString(SMILE_REQUEST_RESULT_TAG)?.let { map.put(SID_RESULT_TAG, it) }
        currentResult.success(map)
        return true
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>?,
        grantResults: IntArray?
    ): Boolean {
        return true
    }

    fun smileCapture(tag: String?, captureType: CaptureType) {
        val builder = SIDCaptureManager.Builder(
            activity,
            captureType,
            SMILE_ID_CARD_REQUEST_CODE
        )
        if (TextUtils.isEmpty(tag)) {
            builder.setTag(tag)
        }
        builder.build().start()
    }

    fun submitJob(
        tag: String?,
        jobType: Int?,
        isProduction: Boolean?,
        partnerParams: HashMap<String, String>?,
        @Nullable idInfo: HashMap<String, String>?,
        @Nullable geoInfo: HashMap<String, String>?
    ) {
        if (TextUtils.isEmpty(tag)) {
            currentResult.error(submitError, "tag is not set", null)
            return
        }
        if (jobType == null) {
            currentResult.error(submitError, "job type is not set", null)
            return
        }

        if (isProduction == null) {
            currentResult.error(submitError, "environment is not set", null)
            return
        }

//        var uploadChannelEvent = EventChannel(flutterEngine.dartExecutor.binaryMessenger,uploadListenerEventChannel)


        val metadata = SIDMetadata()
        val sidNetworkRequest = SIDNetworkRequest(activity)
        sidNetworkRequest.setOnCompleteListener {
//            val params = Arguments.createMap()
//            params.putString("status", "done")
//            sendEvent(reactApplicationContext, "CompleteListener", params);
        }
        sidNetworkRequest.set0nErrorListener {
            currentResult.error(tag, "set0nErrorListener", it)
        }
        sidNetworkRequest.setOnUpdateListener {
//            val params = Arguments.createMap()
//            params.putString("status", it.toString())
//            sendEvent(reactApplicationContext, "UploadListener", params);
        }
        sidNetworkRequest.setOnEnrolledListener {
            if (it != null && it.statusResponse != null) {
                val result = SIDUtil.convertJsonToMap(JSONObject(it.statusResponse.rawJsonString))
                currentResult.success(result)
            } else {
                currentResult.error(submitError, "Oops something went wrong", null)
            }
        }
        sidNetworkRequest.setOnAuthenticatedListener {
            if (it != null && it.statusResponse != null) {
                val result = SIDUtil.convertJsonToMap(JSONObject(it.statusResponse.rawJsonString))
                currentResult.success(result)
            } else {
                currentResult.error(submitError, "Oops something went wrong", null)
            }
        }

        if (idInfo != null && !idInfo.isEmpty()) {
            val sidUserIdInfo = metadata.sidUserIdInfo
            for ((key, value) in idInfo) {
                if (!mainIdInfo.any { mainKey -> mainKey == key }) {
                    sidUserIdInfo.additionalValue(key, value)
                } else {
                    when (key) {
                        "country" -> sidUserIdInfo.countryCode = value
                        "id_type" -> sidUserIdInfo.idType = value
                        "id_number" -> sidUserIdInfo.idNumber = value
                        "first_name" -> sidUserIdInfo.firstName = value
                        "middle_name" -> sidUserIdInfo.middleName = value
                        "last_name" -> sidUserIdInfo.lastName = value
                        "email" -> sidUserIdInfo.email = value
                    }
                }
            }
        }

        if (partnerParams != null && !partnerParams.isEmpty()) {
            val sidPartnerParams = metadata.partnerParams
            for ((key, value) in partnerParams) {
                if (!mainPartnerParams.any { mainKey -> mainKey == key }) {
                    sidPartnerParams.additionalValue(key, value)
                } else {
                    when (key) {
                        "job_id" -> sidPartnerParams.jobId = value
                        "user_id" -> sidPartnerParams.userId = value
                    }
                }
            }
        }

        val geoInfos = GeoInfos()
        if (geoInfo != null && geoInfo.isEmpty()) {
            for ((key, value) in geoInfo) {
                when (key) {
                    "accuracy" -> geoInfos.accuracy = value as Double
                    "altitude" -> geoInfos.altitude = value as Double
                    "latitude" -> geoInfos.latitude = value as Double
                    "longitude" -> geoInfos.longitude = value as Double
                    "lastUpdate" -> geoInfos.lastUpdate = value as String
                    "isGeoPermissionGranted" -> geoInfos.isGeoPermissionGranted =
                        value as Boolean
                }
            }
        }

        val data = SIDNetData(
            activity,
            if (isProduction) SIDNetData.Environment.PROD else SIDNetData.Environment.TEST
        )
        val config = SIDConfig.Builder(activity)
            .setSmileIdNetData(data)
            .setGeoInformation(geoInfos)
            .setSIDMetadata(metadata)
            .setJobType(jobType)
            .setMode(if (jobType == 1 || jobType == 4) SIDConfig.Mode.ENROLL else SIDConfig.Mode.AUTHENTICATION)
            .build(tag)
        activity?.let {
            if (haveNetworkConnection(it)) {
                sidNetworkRequest.submit(config)
            } else {
                currentResult.error(submitError, "Please make sure you are connected", null)
            }
        }
    }


    fun checkPermissions(): Boolean {
        if (permissionGranted(PERMISSIONS)) {
            return true;
        } else {
            ActivityCompat.requestPermissions(
                activity,
                PERMISSIONS,
                PERMISSION_ALL
            )
        }
        return false;
    }

    protected fun permissionGranted(permissions: Array<String>): Boolean {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            for (permission in permissions) {
                if (ActivityCompat.checkSelfPermission(
                        activity,
                        permission!!
                    ) != PackageManager.PERMISSION_GRANTED
                ) {
                    return false
                }
            }
        }
        return true
    }

    fun haveNetworkConnection(context: Context): Boolean {
        val cm = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        if (cm.activeNetworkInfo != null) {
            return cm.activeNetworkInfo!!.isConnected
        } else {
            return false
        }
    }
}

==> smile_flutter-0.0.3.tar.gz: android/src/main/kotlin/com/smileidentity/smile_flutter/SmileFlutterPlugin.kt <==
package com.smileidentity.smile_flutter

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.ConnectivityManager
import android.os.Build
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import androidx.annotation.Nullable
import androidx.core.app.ActivityCompat
import com.rnsmileid.SIDUtil
import com.smileid.smileidui.CaptureType
import com.smileid.smileidui.IntentHelper.SELFIE_REQUEST_RESULT
import com.smileid.smileidui.IntentHelper.SMILE_REQUEST_RESULT_TAG
import com.smileid.smileidui.SIDCaptureManager
import com.smileidentity.libsmileid.core.SIDConfig
import com.smileidentity.libsmileid.core.SIDNetworkRequest
import com.smileidentity.libsmileid.model.GeoInfos
import com.smileidentity.libsmileid.model.SIDMetadata
import com.smileidentity.libsmileid.model.SIDNetData

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.RequestPermissionsResultListener
import org.json.JSONObject

/** SmileFlutterPlugin */
class SmileFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    PluginRegistry.ActivityResultListener, RequestPermissionsResultListener {


    private val uploadListenerEventChannel = "smile_id/upload_listener"
    private val completeListenerEventChannel = "smile_id/complete_listener"

    private val PERMISSION_ALL = 1
    private val SMILE_ID_CARD_REQUEST_CODE = 777
    val SID_RESULT_CODE = "SID_RESULT_CODE"
    val SID_RESULT_MESSAGE = "SID_RESULT_MESSAGE"
    val SID_RESULT_TAG = "SID_RESULT_TAG"

    val mainIdInfo =
        listOf("first_name", "last_name", "middle_name", "country", "id_type", "id_number", "email")
    val mainPartnerParams = listOf("user_id", "job_id")
    val permissionError = "permissionError"
    val submitError = "submitError"

    val permissionErrorMessage = "Does not have required permissions to run this method"

    private var PERMISSIONS = arrayOf(
        Manifest.permission.WRITE_EXTERNAL_STORAGE,
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.CAMERA,
        Manifest.permission.ACCESS_FINE_LOCATION,
        Manifest.permission.READ_PHONE_STATE
    )

    private lateinit var activity: Activity
    private lateinit var activityBinding: ActivityPluginBinding


    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var currentResult: Result

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {

        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "smile_flutter")
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
                return
            }
            "captureSelfie" -> {
                if (!checkPermissions()) {
                    result.error(permissionError, permissionErrorMessage, null)
                    return
                }
                currentResult = result
                smileCapture(call.argument<String>("tag"), CaptureType.SELFIE)
            }
            "captureIDCard" -> {
                if (!checkPermissions()) {
                    result.error(permissionError, permissionErrorMessage, null)
                    return
                }
                currentResult = result
                smileCapture(call.argument<String>("tag"), CaptureType.ID_CAPTURE)
            }
            "captureSelfieAndIDCard" -> {
                if (!checkPermissions()) {
                    result.error(permissionError, permissionErrorMessage, null)
                    return
                }
                currentResult = result
                smileCapture(call.argument<String>("tag"), CaptureType.SELFIE_AND_ID_CAPTURE)
            }
            "submitJob" -> {
                submitJob(
                    call.argument<String>("tag"),
                    call.argument<Int>("jobType"),
                    call.argument<Boolean>("isProduction"),
                    call.argument<HashMap<String, String>>("partnerParams"),
                    call.argument<HashMap<String, String>>("idInfo"),
                    call.argument<HashMap<String, String>>("geoInfo"),
                    result
                )
                return
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }


    //smile id

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityBinding = binding
        activity = binding.activity
        binding.addRequestPermissionsResultListener(this);
        binding.addActivityResultListener(this);
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activityBinding.removeActivityResultListener(this)
        activityBinding.removeRequestPermissionsResultListener(this)
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
        activityBinding.removeRequestPermissionsResultListener(this);
        activityBinding.removeActivityResultListener(this);
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        val map: HashMap<String, Any> = HashMap()
        data?.extras?.getInt(SELFIE_REQUEST_RESULT)?.let { map.put(SID_RESULT_CODE, it) }
        data?.extras?.getString(SMILE_REQUEST_RESULT_TAG)?.let { map.put(SID_RESULT_TAG, it) }
        currentResult.success(map)
        return true
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>?,
        grantResults: IntArray?
    ): Boolean {
        return true
    }

    fun smileCapture(tag: String?, captureType: CaptureType) {
        val builder = SIDCaptureManager.Builder(
            activity,
            captureType,
            SMILE_ID_CARD_REQUEST_CODE
        )
        if (TextUtils.isEmpty(tag)) {
            builder.setTag(tag)
        }
        builder.build().start()
    }

    fun submitJob(
        tag: String?,
        jobType: Int?,
        isProduction: Boolean?,
        partnerParams: HashMap<String, String>?,
        @Nullable idInfo: HashMap<String, String>?,
        @Nullable geoInfo: HashMap<String, String>?,
        channelResult: Result
    ) {
        if (TextUtils.isEmpty(tag)) {
            channelResult.error(submitError, "tag is not set", null)
            return
        }
        if (jobType == null) {
            channelResult.error(submitError, "job type is not set", null)
            return
        }

        if (isProduction == null) {
            channelResult.error(submitError, "environment is not set", null)
            return
        }

//        var uploadChannelEvent = EventChannel(flutterEngine.dartExecutor.binaryMessenger,uploadListenerEventChannel)


        val metadata = SIDMetadata()
        val sidNetworkRequest = SIDNetworkRequest(activity)
        sidNetworkRequest.setOnCompleteListener {
//            val params = Arguments.createMap()
//            params.putString("status", "done")
//            sendEvent(reactApplicationContext, "CompleteListener", params);
        }
        sidNetworkRequest.set0nErrorListener {
            channelResult.error(tag, "set0nErrorListener", it)
        }
        sidNetworkRequest.setOnUpdateListener {
//            val params = Arguments.createMap()
//            params.putString("status", it.toString())
//            sendEvent(reactApplicationContext, "UploadListener", params);
        }
        sidNetworkRequest.setOnEnrolledListener {
            if (it != null && it.statusResponse != null) {
                val result = SIDUtil.convertJsonToMap(JSONObject(it.statusResponse.rawJsonString))
                channelResult.success(result)
            }
        }
        sidNetworkRequest.setOnAuthenticatedListener {
            if (it != null && it.statusResponse != null) {
                val result = SIDUtil.convertJsonToMap(JSONObject(it.statusResponse.rawJsonString))
                channelResult.success(result)
            }
        }

        if (idInfo != null && !idInfo.isEmpty()) {
            val sidUserIdInfo = metadata.sidUserIdInfo
            for ((key, value) in idInfo) {
                if (!mainIdInfo.any { mainKey -> mainKey == key }) {
                    sidUserIdInfo.additionalValue(key, value)
                } else {
                    when (key) {
                        "country" -> sidUserIdInfo.countryCode = value
                        "id_type" -> sidUserIdInfo.idType = value
                        "id_number" -> sidUserIdInfo.idNumber = value
                        "first_name" -> sidUserIdInfo.firstName = value
                        "middle_name" -> sidUserIdInfo.middleName = value
                        "last_name" -> sidUserIdInfo.lastName = value
                        "email" -> sidUserIdInfo.email = value
                    }
                }
            }
        }

        if (partnerParams != null && !partnerParams.isEmpty()) {
            val sidPartnerParams = metadata.partnerParams
            for ((key, value) in partnerParams) {
                if (!mainPartnerParams.any { mainKey -> mainKey == key }) {
                    sidPartnerParams.additionalValue(key, value)
                } else {
                    when (key) {
                        "job_id" -> sidPartnerParams.jobId = value
                        "user_id" -> sidPartnerParams.userId = value
                    }
                }
            }
        }

        val geoInfos = GeoInfos()
        if (geoInfo != null && geoInfo.isEmpty()) {
            for ((key, value) in geoInfo) {
                when (key) {
                    "accuracy" -> geoInfos.accuracy = value as Double
                    "altitude" -> geoInfos.altitude = value as Double
                    "latitude" -> geoInfos.latitude = value as Double
                    "longitude" -> geoInfos.longitude = value as Double
                    "lastUpdate" -> geoInfos.lastUpdate = value as String
                    "isGeoPermissionGranted" -> geoInfos.isGeoPermissionGranted =
                        value as Boolean
                }
            }
        }

        val data = SIDNetData(
            activity,
            if (isProduction) SIDNetData.Environment.PROD else SIDNetData.Environment.TEST
        )
        val config = SIDConfig.Builder(activity)
            .setSmileIdNetData(data)
            .setGeoInformation(geoInfos)
            .setSIDMetadata(metadata)
            .setJobType(jobType)
            .setMode(if (jobType == 1 || jobType == 4) SIDConfig.Mode.ENROLL else SIDConfig.Mode.AUTHENTICATION)
            .build(tag)
        activity?.let {
            if (haveNetworkConnection(it)) {
                sidNetworkRequest.submit(config)
            } else {
                channelResult.error(submitError, "Please make sure you are connected", null)
            }
        }
    }


    fun checkPermissions(): Boolean {
        if (permissionGranted(PERMISSIONS)) {
            return true;
        } else {
            ActivityCompat.requestPermissions(
                activity,
                PERMISSIONS,
                PERMISSION_ALL
            )
        }
        return false;
    }

    protected fun permissionGranted(permissions: Array<String>): Boolean {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            for (permission in permissions) {
                if (ActivityCompat.checkSelfPermission(
                        activity,
                        permission!!
                    ) != PackageManager.PERMISSION_GRANTED
                ) {
                    return false
                }
            }
        }
        return true
    }

    fun haveNetworkConnection(context: Context): Boolean {
        val cm = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        if (cm.activeNetworkInfo != null) {
            return cm.activeNetworkInfo!!.isConnected
        } else {
            return false
        }
    }
}
